@@ -30,6 +30,7 @@
 #define HAVE_DLFCN_H 1
 #define HAVE_EXECINFO_H 1
 #define HAVE_FCNTL_H 1
+#define HAVE_FCNTL_DIRECT 1
 #define HAVE_FENV_H 1
 #define HAVE_FLOAT_H 1
 #define HAVE_FNMATCH_H 1
@@ -85,7 +86,7 @@
 #define HAVE_SYS_STAT_H 1
 /* #undef HAVE_SYS_STREAM_H */
 #define HAVE_SYS_SYSCALL_H 1
-#define HAVE_SYS_TIMEB_H 1
+/* #undef HAVE_SYS_TIMEB_H */
 #define HAVE_SYS_TIMES_H 1
 #define HAVE_SYS_TIME_H 1
 #define HAVE_SYS_TYPES_H 1
@@ -151,6 +152,7 @@
 #define HAVE_GETCWD 1
 #define HAVE_GETHOSTBYADDR_R 1
 /* #undef HAVE_GETHRTIME */
+#define HAVE_GETPAGESIZE 1
 /* #undef HAVE_GETPAGESIZES */
 #define HAVE_GETPASS 1
 /* #undef HAVE_GETPASSPHRASE */
@@ -195,7 +197,6 @@
 #define HAVE_PTHREAD_ATTR_GETSTACKSIZE 1
 #define HAVE_PTHREAD_ATTR_SETSCOPE 1
 #define HAVE_PTHREAD_ATTR_SETSTACKSIZE 1
-#define HAVE_PTHREAD_GETATTR_NP 1
 /* #undef HAVE_PTHREAD_CONDATTR_CREATE */
 #define HAVE_PTHREAD_GETAFFINITY_NP 1
 #define HAVE_PTHREAD_KEY_DELETE 1
@@ -402,27 +403,38 @@
 #define SIGNAL_WITH_VIO_CLOSE 1
 
 /* Windows stuff, mostly functions, that have Posix analogs but named differently */
-#ifdef _WIN32
-#define S_IROTH _S_IREAD
-#define S_IFIFO _S_IFIFO
-#define SIGQUIT SIGTERM
-#define SIGPIPE SIGINT
-#define sigset_t int
-#define mode_t int
-#define popen _popen
-#define pclose _pclose
-#define ssize_t SSIZE_T
-#define strcasecmp _stricmp
-#define strncasecmp _strnicmp
-#define strtok_r strtok_s
+/* #undef S_IROTH */
+/* #undef S_IFIFO */
+/* #undef IPPROTO_IPV6 */
+/* #undef IPV6_V6ONLY */
+/* #undef sigset_t */
+/* #undef mode_t */
+/* #undef SIGQUIT */
+/* #undef SIGPIPE */
+/* #undef popen */
+/* #undef pclose */
+/* #undef ssize_t */
+/* #undef strcasecmp */
+/* #undef strncasecmp */
+/* #undef snprintf */
+/* #undef strtok_r */
+/* #undef strtoll */
+/* #undef strtoull */
+/* #undef vsnprintf */
+#if defined(_MSC_VER) && (_MSC_VER > 1800)
 #define tzname _tzname
 #define P_tmpdir "C:\\TEMP"
+#endif
+#if defined(_MSC_VER) && (_MSC_VER > 1310)
+# define HAVE_SETENV
 #define setenv(a,b,c) _putenv_s(a,b)
+#endif
+#define PSAPI_VERSION 1     /* for GetProcessMemoryInfo() */
 
-#define HAVE_SETENV
+/* We don't want the min/max macros */
+#ifdef _WIN32
 #define NOMINMAX 1
-#define PSAPI_VERSION 2     /* for GetProcessMemoryInfo() */
-#endif /* _WIN32 */
+#endif
 
 /*
   MySQL features
@@ -446,11 +458,6 @@
 /* This should mean case insensitive file system */
 /* #undef FN_NO_CASE_SENSE */
 
-/* Whether an anonymous private mapping is unaccessible after
-madvise(MADV_DONTNEED) or madvise(MADV_FREE) or similar has been invoked;
-this is the case with Microsoft Windows VirtualFree(MEM_DECOMMIT) */
-/* #undef HAVE_UNACCESSIBLE_AFTER_MEM_DECOMMIT */
-
 #define HAVE_CHARSET_armscii8 1
 #define HAVE_CHARSET_ascii 1
 #define HAVE_CHARSET_big5 1
@@ -491,9 +498,8 @@ this is the case with Microsoft Windows VirtualFree(MEM_DECOMMIT) */
 #define HAVE_CHARSET_utf32 1
 #define HAVE_UCA_COLLATIONS 1
 #define HAVE_COMPRESS 1
-/* #undef HAVE_EncryptAes128Ctr */
+#define HAVE_EncryptAes128Ctr 1
 /* #undef HAVE_EncryptAes128Gcm */
-#define HAVE_des 1
 
 /*
   Stuff that always need to be defined (compile breaks without it)
@@ -524,18 +530,18 @@ this is the case with Microsoft Windows VirtualFree(MEM_DECOMMIT) */
 
 /* #undef SO_EXT */
 
-#define MYSQL_VERSION_MAJOR 10
-#define MYSQL_VERSION_MINOR 11
-#define MYSQL_VERSION_PATCH 15
+#define MYSQL_VERSION_MAJOR 11
+#define MYSQL_VERSION_MINOR 2
+#define MYSQL_VERSION_PATCH 3
 #define MYSQL_VERSION_EXTRA ""
 
 #define PACKAGE "mysql"
 #define PACKAGE_BUGREPORT ""
 #define PACKAGE_NAME "MySQL Server"
-#define PACKAGE_STRING "MySQL Server 10.11.15"
+#define PACKAGE_STRING "MySQL Server 11.2.3"
 #define PACKAGE_TARNAME "mysql"
-#define PACKAGE_VERSION "10.11.15"
-#define VERSION "10.11.15"
+#define PACKAGE_VERSION "11.2.3"
+#define VERSION "11.2.3"
 #define PROTOCOL_VERSION 10
 #define PCRE2_CODE_UNIT_WIDTH 8
 
@@ -549,6 +555,7 @@ this is the case with Microsoft Windows VirtualFree(MEM_DECOMMIT) */
 #ifndef EMBEDDED_LIBRARY
 /* #undef WSREP_INTERFACE_VERSION */
 #define WITH_WSREP 1
+#define WSREP_PROC_INFO 1
 #endif
 
 #if !defined(__STDC_FORMAT_MACROS)
